/************************************************************************/
/* TaxSolve_OH_IT1040_2008.c - 						*/
/* Copyright (C) 2009 - Aston Roberts					*/
/* 									*/
/* Compile:   gcc taxsolve_OH_IT1040_2008.c -o taxsolve_OH_IT1040_2008	*/
/* Run:	      ./taxsolve_OH_IT1040_2008  OH_IT1040_2008.dat 		*/
/* 									*/
/* GNU Public License - GPL:						*/
/* This program is free software; you can redistribute it and/or	*/
/* modify it under the terms of the GNU General Public License as	*/
/* published by the Free Software Foundation; either version 2 of the	*/
/* License, or (at your option) any later version.			*/
/* 									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU	*/
/* General Public License for more details.				*/
/* 									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA		*/
/* 02111-1307 USA							*/
/* 									*/
/* Aston Roberts 1-2-2004	aston_roberts@yahoo.com			*/
/* Updates by:  Aston Roberts 1-18-2005					*/
/* Updates by:  Aston Roberts 2-21-2006					*/
/* Updates by:  Aston Roberts 2-9-2008					*/
/* Updates by:  Aston Roberts 1-10-2009					*/
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include "taxsolve_routines.c"

double thisversion=6.0;

#define SINGLE 		        1
#define MARRIED_FILLING_JOINTLY 2
#define MARRIED_FILLING_SEPARAT 3
#define HEAD_OF_HOUSEHOLD       1

double TaxRateFunction( double x, int status )
{
 if (x <  5000.0) return x * 0.00618; else
 if (x < 10000.0) return  30.90 + (x-5000.0) * 0.01236; else
 if (x < 15000.0) return  92.70 + (x-10000.0) * 0.02473; else
 if (x < 20000.0) return 216.35 + (x-15000.0) * 0.03091; else
 if (x < 40000.0) return 370.90 + (x-20000.0) * 0.03708; else
 if (x < 80000.0) return 1112.50 + (x-40000.0) * 0.04327; else
 if (x < 100000.0) return 2843.30 + (x-80000.0) * 0.04945; else
 if (x < 200000.0) return 3832.30 + (x-100000.0) * 0.05741; else
 return 9573.30 + (x-200000.0) * 0.06240; 
}



/*----------------------------------------------------------------------------*/

int main( int argc, char *argv[] )
{
 int i, j, k;
 char word[1000], outfname[1000];
 int status=0, exemptions=0;
 time_t now;
 double value;

 /* Intercept any command-line arguments. */
 printf("OH IT1040 2008 - v%3.1f\n", thisversion);
 i = 1;  k=1;
 while (i < argc)
 {
  if (strcmp(argv[i],"-verbose")==0)  verbose = 1;
  else
  if (k==1)
   {
    infile = fopen(argv[i],"r");
    if (infile==0) {printf("ERROR: Parameter file '%s' could not be opened.\n", argv[i]); exit(1);}
    k = 2;
    /* Base name of output file on input file. */
    strcpy(outfname,argv[i]);
    j = strlen(outfname)-1;
    while ((j>=0) && (outfname[j]!='.')) j--;
    if (j<0) strcat(outfname,".out"); else strcpy(&(outfname[j]),".out");
    outfile = fopen(outfname,"w");
    if (outfile==0) {printf("ERROR: Output file '%s' could not be opened.\n", outfname); exit(1);}
    printf("Writing results to file:  %s\n", outfname);
   }
  else {printf("Unknown command-line parameter '%s'\n", argv[i]); exit(1);}
  i = i + 1;
 }

 if (infile==0) {printf("Error: No input file on command line.\n"); exit(1);}

 /* Pre-initialize all lines to zeros. */
 for (i=0; i<MAX_LINES; i++) 
  {
   L[i] = 0.0;
  }

 /* Accept parameters from input file. */
 /* Expect  OH IT1040 lines, something like:
	Title:  OH IT1040 1999 Return
	L1		{Wages}
*/


 /* Accept Form's "Title" line, and put out with date-stamp for records. */
 read_line( infile, word );
 now = time(0);
 fprintf(outfile,"\n%s		%s\n", word, ctime( &now ));

 /* get_parameter(infile, kind, x, emssg ) */
 get_parameter( infile, 's', word, "Status" );
 get_parameter( infile, 'l', word, "Status ?");
 if (strncasecmp(word,"Single",4)==0) status = SINGLE; else
 if (strncasecmp(word,"Married/Joint",13)==0) status = MARRIED_FILLING_JOINTLY; else
 if (strncasecmp(word,"Married/Sep",11)==0) status = MARRIED_FILLING_SEPARAT; else
 if (strncasecmp(word,"Head_of_House",4)==0) status = HEAD_OF_HOUSEHOLD;
 else
  { 
   printf("Error: unrecognized status '%s'. Must be: Single, Married/joint, Married/sep, Head_of_house.\nExiting.\n", word); 
   exit(0); 
  }
 fprintf(outfile,"Status = %s (%d)\n", word, status);

 GetLine( "L1", &L[1] );	/* Federal Adjusted Gross Income */

 get_parameter( infile, 's', word, "Exemptions" );	/* Exemptions, self/depend. */
 get_parameters( infile, 'i', &exemptions, "Exemptions"); 

 GetLine( "L11\%", &L[11] );	/* Joint Filing Credit Percentage (%) */
 L[11] = 0.01 * L[11];
 // GetLine( "L13", &L[13] );	/* Resident/Nonres/Part-Yr Credits (Sch. C/D) & Nonrefund. Bus. Credits (Sch. E) */
 GetLine( "L15", &L[15] );	/* Manufacturing equipment grant. */
 GetLine( "L17", &L[17] );	/* Interest Penalty , Underpaid Estimated Tax */
 GetLine( "L18", &L[18] );	/* Unpaid Ohio Use Tax */
 GetLine( "L20", &L[20] );	/* Ohio Tax Withheld (box 17 on your W-2) */
 GetLine( "L21", &L[21] );	/* Ohio Estimated Tax, IT 40P Payments for 2008, ... */
 GetLine( "L22", &L[22] );	/* Refundable Business Jobs + Refundable Pass-through Entity */

 /* Additions - add to extent not included in fed adjusted gross income (line 1). */
 GetLine( "L31", &L[31] );	/* Non-Ohio state or local gov't interest, dividends. */
 GetLine( "L32", &L[32] );	/* Pass-through Entity addback. */

 get_parameter( infile, 's', word, "33a" );	/* Fed int+div subject to state tax & misc. fed adj. */
 get_parameters( infile, 'f', &value, "33a");
 L[33] = value;

 get_parameter( infile, 's', word, "33b" );	/* Reimbursed college tuit. fees deducted prev yrs. */
 get_parameters( infile, 'f', &value, "33b");
 L[33] = L[33] + value;

 get_parameter( infile, 's', word, "33c" );	/* Losses, sale of Ohio Public Obligations. */
 get_parameters( infile, 'f', &value, "33c");
 L[33] = L[33] + value;

 get_parameter( infile, 's', word, "33d" );	/* Nonmedical withdrawals OH Med. Savings Acct. */
 get_parameters( infile, 'f', &value, "33d");
 L[33] = L[33] + value;

 get_parameter( infile, 's', word, "33e" );	/* Reimb. exp. prev. deducted, if reimb. not in FAGI .*/
 get_parameters( infile, 'f', &value, "33e");
 L[33] = L[33] + value;

 get_parameter( infile, 's', word, "33f" );	/* Non-edu expenses from College Savings Acct. */
 get_parameters( infile, 'f', &value, "33f");
 L[33] = L[33] + value;

 get_parameter( infile, 's', word, "33g" );	/* Depreciation adjust, IRC 168(k) + 179. */
 get_parameters( infile, 'f', &value, "33g");
 L[33] = L[33] + value;

 /* Deductions - see limitations in instructions. */
 GetLine( "L35", &L[35] );	/* Ded. fed int + div exempt from state taxation. */
 GetLine( "L36", &L[36] );	/* Ded. comp. earned in OH by non-residents. */
 GetLine( "L37", &L[37] );	/* Military pay. */
 GetLine( "L38", &L[38] );	/* Ded. state/municipal income tax overpayments. */
 GetLine( "L39", &L[39] );	/* Ded. disab. & survivor benef. */
 GetLine( "L40", &L[40] );	/* Ded. qualifying soc. sec. + railroad benefits. */ 
 GetLine( "L41", &L[41] );	/* Ded. contrib CollegeAdvantage 529 plan. */ 
 GetLine( "L42", &L[42] );	/* Ded. Ohio National Guard reimbursements */
 GetLine( "L43", &L[43] );	/* Ded. Un-subsidized health insurance ... */
 GetLine( "L44", &L[44] );	/* Ded. deposits into Medical Savings Acct ... */
 
 /* Other. Check all that apply: */
 get_parameter( infile, 's', word, "45a" );	/* Wage & salary expense not deducted */
 get_parameters( infile, 'f', &value, "45a");
 L[45] = value;
 get_parameter( infile, 's', word, "45b" );	/* Interest income from Ohio Public Obligations */
 get_parameters( infile, 'f', &value, "45b");
 L[45] = L[45] + value;
 get_parameter( infile, 's', word, "45c" );	/* Refund or reimbursements shown on federal Form 1040, line 21, */
 get_parameters( infile, 'f', &value, "45c");
 L[45] = L[45] + value;
 get_parameter( infile, 's', word, "45d" );	/* Repayment of income reported in a prior year */
 get_parameters( infile, 'f', &value, "45d");
 L[45] = L[45] + value;
 get_parameter( infile, 's', word, "45e" );	/* Amount contributed to an Individual Development Account */
 get_parameters( infile, 'f', &value, "45e");
 L[45] = L[45] + value;
 get_parameter( infile, 's', word, "45f" );	/* Depreciation expense adjustment for IRC sections 168(k) and 179 */
 get_parameters( infile, 'f', &value, "45f");
 L[45] = L[45] + value;
 get_parameter( infile, 's', word, "45g" );	/* Military retirement income */
 get_parameters( infile, 'f', &value, "45g");
 L[45] = L[45] + value;

 GetLine( "L48", &L[48] );	/* Retirement Income Credit (see instructions for credit table) (Limit $200) */
 if (L[48] > 200.0) L[48] = 200.0;
 GetLine( "L49", &L[49] );	/* Senior Citizen Credit (Limit $50 per return) */
 if (L[49] > 50.0) L[49] = 50.0;
 GetLine( "L50", &L[50] );	/* Lump Sum Distribution Credit (you must be 65 years of age or older to claim this credit) */
 GetLine( "L51", &L[51] );	/* Child and Dependent Care Credit (see instructions and worksheet) */
 GetLine( "L52", &L[52] );	/* Lump Sum Retirement Credit */
 GetLine( "L54", &L[54] );	/* Job Training Credit (see instructions & worksheet) (Limit $500 single; $1,000 joint, if both spouses qualify) */
 if (L[54] > 500.0) L[54] = 500.0;
 GetLine( "L55", &L[55] );	/* Ohio Political Contributions Credit (Limit $50 single; $100 joint) */
 if (L[55] > 50.0) L[55] = 50.0;
 GetLine( "L56", &L[56] );	/* Ohio Adoption Credit (Limit $1500 per adoption) */

 GetLine( "L58", &L[58] );	/* Portion of line 3 subjected to tax by other states. */
 GetLine( "L61", &L[61] );	/* 2008 income tax paid to other states ... */

 /* For non-residents only. */
 GetLine( "L63", &L[63] );	/* Portion Ohio Adj. Gross Income not earned in Oh. */

 GetLine( "L66", &L[66] );	/* Nonrefundable Business Credits, Sched-E line 11 */




 /* Do Calculations. */

 L[34] = L[31] + L[32] + L[33];

 for (j=35; j<=45; j++)
  L[46] = L[46] + L[j];

 L[47] = L[34] - L[46];
 L[2] = L[47];

 L[3] = L[1] - L[2];
 L[4] = 1500.0 * exemptions;
 L[5] = NotLessThanZero( L[3] - L[4] );
 L[6] = TaxRateFunction( L[5], status );

 if (L[5] <= 10000.0) L[53] = 93.0;

 for (j=48; j<=56; j++)
  L[57] = L[57] + L[j];
 L[7] = L[57];

 L[8] = NotLessThanZero( L[6] - L[7] );
 L[9] = 20.0 * exemptions;
 L[10] = NotLessThanZero( L[8] - L[9] );
 L[11] = L[11] * L[10];
 if (L[11] > 650.0) L[11] = 650.0;
 L[12] = NotLessThanZero( L[10] - L[11] );

 L[59] = L[3];
 L[60] = (L[58] / L[59]) * L[12];
 L[62] = smallerof( L[60], L[62] );
 L[67] = L[62];

 L[14] = NotLessThanZero( L[12] - L[13] );
 L[16] = NotLessThanZero( L[14] - L[15] );
 L[19] = L[16] + L[17] + L[18];
 L[23] = L[20] + L[21] + L[22];

 L[64] = L[3];
 L[65] = (L[63] / L[64]) * L[12];
 L[68] = L[65];

 L[67] = L[62];
 L[69] = L[66] + L[67] + L[68];
 L[13] = L[69];


 /* Output the Results. */

 showline(1);
 showline(2);
 showline(3);
 fprintf(outfile,"Exemptions = %d\n", exemptions );
 for (j=4; j<=18; j++)
  showline(j);
 showline_wmsg(19, "Total Ohio Tax");
 showline(20);
 showline(21);
 showline(22);
 showline_wmsg(23, "Total Payments");
 if (L[23] > L[19])
  {
   L[24] = L[19] - L[23];
   showline_wmsg(24, "AMOUNT YOU OVERPAID");
  }
 else
  {
   L[30] = L[23] - L[19];
   showline_wmsg(30, "AMOUNT YOU OWE");
  }
 showline(31);
 showline(32);
 showline(33);
 showline_wmsg(34,"Total additions");
 for (j=35; j<=45; j++)
  showline(j);
 showline_wmsg(46,"Total Deductions");
 showline(47);
 for (j=48; j<=56; j++)
  showline(j);
 showline_wmsg(57,"Total Credits");
 for (j=58; j<=60; j++)
  showline(j);
 fprintf(outfile," (L60 [%3.1f%%] )\n", 100.0 * L[58] / L[59] );
 for (j=61; j<=69; j++)
  showline(j);

 fclose(infile);
 fclose(outfile);
 Display_File( outfname );
 printf("\nResults written to file:  %s\n", outfname);
 return 0;
}
